# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from airflow import DAG
from airflow.providers.cncf.kubernetes.operators import kubernetes_pod
from airflow.providers.google.cloud.transfers import gcs_to_bigquery

default_args = {
    "owner": "Google",
    "depends_on_past": False,
    "start_date": "2021-03-01",
}


with DAG(
    dag_id="census_bureau_acs.cbsa_2019_1yr",
    default_args=default_args,
    max_active_runs=1,
    schedule_interval="@once",
    catchup=False,
    default_view="graph",
) as dag:

    # Run CSV transform within kubernetes pod
    cbsa_2019_1yr_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="cbsa_2019_1yr_transform_csv",
        startup_timeout_seconds=600,
        name="cbsa_2019_1yr",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "https://api.census.gov/data/2019/acs/acs~year_report~?get=NAME,~group_id~_~row_position~E&for=~api_naming_convention~:*&key=550e53635053be51754b09b5e9f5009c94aa0586",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "metropolitan%20statistical%20area/micropolitan%20statistical%20area",
            "TARGET_FILE": "files/data_output.csv",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "data/census_bureau_acs/cbsa_2019_1yr/data_output.csv",
            "PIPELINE_NAME": "cbsa_2019_1yr",
            "GEOGRAPHY": "cbsa",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL": '["combined_statistical_area"]',
            "RENAME_MAPPINGS": '{"0":"name", "1":"KPI_Value", "2":"combined_statistical_area"}',
            "CSV_HEADERS": '["geo_id","aggregate_travel_time_to_work","amerindian_including_hispanic","amerindian_pop","armed_forces","asian_including_hispanic","asian_male_45_54","asian_male_55_64","asian_pop","associates_degree","bachelors_degree","bachelors_degree_2","bachelors_degree_or_higher_25_64","black_including_hispanic","black_male_45_54","black_male_55_64","black_pop","children","children_in_single_female_hh","civilian_labor_force","commute_10_14_mins","commute_15_19_mins","commute_20_24_mins","commute_25_29_mins","commute_30_34_mins","commute_35_39_mins","commute_35_44_mins","commute_40_44_mins","commute_45_59_mins","commute_5_9_mins","commute_60_89_mins","commute_60_more_mins","commute_90_more_mins","commute_less_10_mins","commuters_16_over","commuters_by_bus","commuters_by_car_truck_van","commuters_by_carpool","commuters_by_public_transportation","commuters_by_subway_or_elevated","commuters_drove_alone","different_house_year_ago_different_city","different_house_year_ago_same_city","dwellings_10_to_19_units","dwellings_1_units_attached","dwellings_1_units_detached","dwellings_20_to_49_units","dwellings_2_units","dwellings_3_to_4_units","dwellings_50_or_more_units","dwellings_5_to_9_units","employed_agriculture_forestry_fishing_hunting_mining","employed_arts_entertainment_recreation_accommodation_food","employed_construction","employed_education_health_social","employed_finance_insurance_real_estate","employed_information","employed_manufacturing","employed_other_services_not_public_admin","employed_pop","employed_public_administration","employed_retail_trade","employed_science_management_admin_waste","employed_transportation_warehousing_utilities","employed_wholesale_trade","families_with_young_children","family_households","father_in_labor_force_one_parent_families_with_young_children","father_one_parent_families_with_young_children","female_10_to_14","female_15_to_17","female_18_to_19","female_20","female_21","female_22_to_24","female_25_to_29","female_30_to_34","female_35_to_39","female_40_to_44","female_45_to_49","female_50_to_54","female_55_to_59","female_5_to_9","female_60_to_61","female_62_to_64","female_65_to_66","female_67_to_69","female_70_to_74","female_75_to_79","female_80_to_84","female_85_and_over","female_female_households","female_pop","female_under_5","four_more_cars","gini_index","graduate_professional_degree","group_quarters","high_school_diploma","high_school_including_ged","hispanic_any_race","hispanic_male_45_54","hispanic_male_55_64","hispanic_pop","households","households_public_asst_or_food_stamps","households_retirement_income","housing_built_1939_or_earlier","housing_built_2000_to_2004","housing_built_2005_or_later","housing_units","housing_units_renter_occupied","in_grades_1_to_4","in_grades_5_to_8","in_grades_9_to_12","in_school","in_undergrad_college","income_100000_124999","income_10000_14999","income_125000_149999","income_150000_199999","income_15000_19999","income_200000_or_more","income_20000_24999","income_25000_29999","income_30000_34999","income_35000_39999","income_40000_44999","income_45000_49999","income_50000_59999","income_60000_74999","income_75000_99999","income_less_10000","income_per_capita","less_one_year_college","less_than_high_school_graduate","male_10_to_14","male_15_to_17","male_18_to_19","male_20","male_21","male_22_to_24","male_25_to_29","male_30_to_34","male_35_to_39","male_40_to_44","male_45_64_associates_degree","male_45_64_bachelors_degree","male_45_64_grade_9_12","male_45_64_graduate_degree","male_45_64_high_school","male_45_64_less_than_9_grade","male_45_64_some_college","male_45_to_49","male_45_to_64","male_50_to_54","male_55_to_59","male_5_to_9","male_60_to_61","male_62_to_64","male_65_to_66","male_67_to_69","male_70_to_74","male_75_to_79","male_80_to_84","male_85_and_over","male_male_households","male_pop","male_under_5","management_business_sci_arts_employed","married_households","masters_degree","median_age","median_income","median_rent","median_year_structure_built","million_dollar_housing_units","mobile_homes","mortgaged_housing_units","no_car","no_cars","nonfamily_households","not_hispanic_pop","not_in_labor_force","not_us_citizen_pop","occupation_management_arts","occupation_natural_resources_construction_maintenance","occupation_production_transportation_material","occupation_sales_office","occupation_services","occupied_housing_units","one_car","one_parent_families_with_young_children","one_year_more_college","other_race_pop","owner_occupied_housing_units","owner_occupied_housing_units_lower_value_quartile","owner_occupied_housing_units_median_value","owner_occupied_housing_units_upper_value_quartile","percent_income_spent_on_rent","pop_16_over","pop_25_64","pop_25_years_over","pop_5_years_over","pop_determined_poverty_status","pop_in_labor_force","population_1_year_and_over","population_3_years_over","poverty","rent_10_to_15_percent","rent_15_to_20_percent","rent_20_to_25_percent","rent_25_to_30_percent","rent_30_to_35_percent","rent_35_to_40_percent","rent_40_to_50_percent","rent_burden_not_computed","rent_over_50_percent","rent_under_10_percent","renter_occupied_housing_units_paying_cash_median_gross_rent","sales_office_employed","some_college_and_associates_degree","speak_only_english_at_home","speak_spanish_at_home","speak_spanish_at_home_low_english","three_cars","total_pop","two_cars","two_or_more_races_pop","two_parent_families_with_young_children","two_parents_father_in_labor_force_families_with_young_children","two_parents_in_labor_force_families_with_young_children","two_parents_mother_in_labor_force_families_with_young_children","two_parents_not_in_labor_force_families_with_young_children","unemployed_pop","vacant_housing_units","vacant_housing_units_for_rent","vacant_housing_units_for_sale","walked_to_work","white_including_hispanic","white_male_45_54","white_male_55_64","white_pop","worked_at_home","workers_16_and_over"]',
        },
        resources={
            "request_memory": "2G",
            "request_cpu": "1",
            "request_ephemeral_storage": "10G",
        },
    )

    # Task to load CSV data to a BigQuery table
    load_cbsa_2019_1yr_to_bq = gcs_to_bigquery.GCSToBigQueryOperator(
        task_id="load_cbsa_2019_1yr_to_bq",
        bucket="{{ var.value.composer_bucket }}",
        source_objects=["data/census_bureau_acs/cbsa_2019_1yr/data_output.csv"],
        source_format="CSV",
        destination_project_dataset_table="census_bureau_acs.cbsa_2019_1yr",
        skip_leading_rows=1,
        allow_quoted_newlines=True,
        write_disposition="WRITE_TRUNCATE",
        schema_fields=[
            {"name": "geo_id", "type": "string", "mode": "nullable"},
            {
                "name": "aggregate_travel_time_to_work",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "amerindian_including_hispanic",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "amerindian_pop", "type": "float", "mode": "nullable"},
            {"name": "armed_forces", "type": "float", "mode": "nullable"},
            {"name": "asian_including_hispanic", "type": "float", "mode": "nullable"},
            {"name": "asian_male_45_54", "type": "float", "mode": "nullable"},
            {"name": "asian_male_55_64", "type": "float", "mode": "nullable"},
            {"name": "asian_pop", "type": "float", "mode": "nullable"},
            {"name": "associates_degree", "type": "float", "mode": "nullable"},
            {"name": "bachelors_degree", "type": "float", "mode": "nullable"},
            {"name": "bachelors_degree_2", "type": "float", "mode": "nullable"},
            {
                "name": "bachelors_degree_or_higher_25_64",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "black_including_hispanic", "type": "float", "mode": "nullable"},
            {"name": "black_male_45_54", "type": "float", "mode": "nullable"},
            {"name": "black_male_55_64", "type": "float", "mode": "nullable"},
            {"name": "black_pop", "type": "float", "mode": "nullable"},
            {"name": "children", "type": "float", "mode": "nullable"},
            {
                "name": "children_in_single_female_hh",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "civilian_labor_force", "type": "float", "mode": "nullable"},
            {"name": "commute_10_14_mins", "type": "float", "mode": "nullable"},
            {"name": "commute_15_19_mins", "type": "float", "mode": "nullable"},
            {"name": "commute_20_24_mins", "type": "float", "mode": "nullable"},
            {"name": "commute_25_29_mins", "type": "float", "mode": "nullable"},
            {"name": "commute_30_34_mins", "type": "float", "mode": "nullable"},
            {"name": "commute_35_39_mins", "type": "float", "mode": "nullable"},
            {"name": "commute_35_44_mins", "type": "float", "mode": "nullable"},
            {"name": "commute_40_44_mins", "type": "float", "mode": "nullable"},
            {"name": "commute_45_59_mins", "type": "float", "mode": "nullable"},
            {"name": "commute_5_9_mins", "type": "float", "mode": "nullable"},
            {"name": "commute_60_89_mins", "type": "float", "mode": "nullable"},
            {"name": "commute_60_more_mins", "type": "float", "mode": "nullable"},
            {"name": "commute_90_more_mins", "type": "float", "mode": "nullable"},
            {"name": "commute_less_10_mins", "type": "float", "mode": "nullable"},
            {"name": "commuters_16_over", "type": "float", "mode": "nullable"},
            {"name": "commuters_by_bus", "type": "float", "mode": "nullable"},
            {"name": "commuters_by_car_truck_van", "type": "float", "mode": "nullable"},
            {"name": "commuters_by_carpool", "type": "float", "mode": "nullable"},
            {
                "name": "commuters_by_public_transportation",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "commuters_by_subway_or_elevated",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "commuters_drove_alone", "type": "float", "mode": "nullable"},
            {
                "name": "different_house_year_ago_different_city",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "different_house_year_ago_same_city",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "dwellings_10_to_19_units", "type": "float", "mode": "nullable"},
            {"name": "dwellings_1_units_attached", "type": "float", "mode": "nullable"},
            {"name": "dwellings_1_units_detached", "type": "float", "mode": "nullable"},
            {"name": "dwellings_20_to_49_units", "type": "float", "mode": "nullable"},
            {"name": "dwellings_2_units", "type": "float", "mode": "nullable"},
            {"name": "dwellings_3_to_4_units", "type": "float", "mode": "nullable"},
            {"name": "dwellings_50_or_more_units", "type": "float", "mode": "nullable"},
            {"name": "dwellings_5_to_9_units", "type": "float", "mode": "nullable"},
            {
                "name": "employed_agriculture_forestry_fishing_hunting_mining",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "employed_arts_entertainment_recreation_accommodation_food",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "employed_construction", "type": "float", "mode": "nullable"},
            {
                "name": "employed_education_health_social",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "employed_finance_insurance_real_estate",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "employed_information", "type": "float", "mode": "nullable"},
            {"name": "employed_manufacturing", "type": "float", "mode": "nullable"},
            {
                "name": "employed_other_services_not_public_admin",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "employed_pop", "type": "float", "mode": "nullable"},
            {
                "name": "employed_public_administration",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "employed_retail_trade", "type": "float", "mode": "nullable"},
            {
                "name": "employed_science_management_admin_waste",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "employed_transportation_warehousing_utilities",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "employed_wholesale_trade", "type": "float", "mode": "nullable"},
            {
                "name": "families_with_young_children",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "family_households", "type": "float", "mode": "nullable"},
            {
                "name": "father_in_labor_force_one_parent_families_with_young_children",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "father_one_parent_families_with_young_children",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "female_10_to_14", "type": "float", "mode": "nullable"},
            {"name": "female_15_to_17", "type": "float", "mode": "nullable"},
            {"name": "female_18_to_19", "type": "float", "mode": "nullable"},
            {"name": "female_20", "type": "float", "mode": "nullable"},
            {"name": "female_21", "type": "float", "mode": "nullable"},
            {"name": "female_22_to_24", "type": "float", "mode": "nullable"},
            {"name": "female_25_to_29", "type": "float", "mode": "nullable"},
            {"name": "female_30_to_34", "type": "float", "mode": "nullable"},
            {"name": "female_35_to_39", "type": "float", "mode": "nullable"},
            {"name": "female_40_to_44", "type": "float", "mode": "nullable"},
            {"name": "female_45_to_49", "type": "float", "mode": "nullable"},
            {"name": "female_50_to_54", "type": "float", "mode": "nullable"},
            {"name": "female_55_to_59", "type": "float", "mode": "nullable"},
            {"name": "female_5_to_9", "type": "float", "mode": "nullable"},
            {"name": "female_60_to_61", "type": "float", "mode": "nullable"},
            {"name": "female_62_to_64", "type": "float", "mode": "nullable"},
            {"name": "female_65_to_66", "type": "float", "mode": "nullable"},
            {"name": "female_67_to_69", "type": "float", "mode": "nullable"},
            {"name": "female_70_to_74", "type": "float", "mode": "nullable"},
            {"name": "female_75_to_79", "type": "float", "mode": "nullable"},
            {"name": "female_80_to_84", "type": "float", "mode": "nullable"},
            {"name": "female_85_and_over", "type": "float", "mode": "nullable"},
            {"name": "female_female_households", "type": "float", "mode": "nullable"},
            {"name": "female_pop", "type": "float", "mode": "nullable"},
            {"name": "female_under_5", "type": "float", "mode": "nullable"},
            {"name": "four_more_cars", "type": "float", "mode": "nullable"},
            {"name": "gini_index", "type": "float", "mode": "nullable"},
            {
                "name": "graduate_professional_degree",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "group_quarters", "type": "float", "mode": "nullable"},
            {"name": "high_school_diploma", "type": "float", "mode": "nullable"},
            {"name": "high_school_including_ged", "type": "float", "mode": "nullable"},
            {"name": "hispanic_any_race", "type": "float", "mode": "nullable"},
            {"name": "hispanic_male_45_54", "type": "float", "mode": "nullable"},
            {"name": "hispanic_male_55_64", "type": "float", "mode": "nullable"},
            {"name": "hispanic_pop", "type": "float", "mode": "nullable"},
            {"name": "households", "type": "float", "mode": "nullable"},
            {
                "name": "households_public_asst_or_food_stamps",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "households_retirement_income",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "housing_built_1939_or_earlier",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "housing_built_2000_to_2004", "type": "float", "mode": "nullable"},
            {
                "name": "housing_built_2005_or_later",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "housing_units", "type": "float", "mode": "nullable"},
            {
                "name": "housing_units_renter_occupied",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "in_grades_1_to_4", "type": "float", "mode": "nullable"},
            {"name": "in_grades_5_to_8", "type": "float", "mode": "nullable"},
            {"name": "in_grades_9_to_12", "type": "float", "mode": "nullable"},
            {"name": "in_school", "type": "float", "mode": "nullable"},
            {"name": "in_undergrad_college", "type": "float", "mode": "nullable"},
            {"name": "income_100000_124999", "type": "float", "mode": "nullable"},
            {"name": "income_10000_14999", "type": "float", "mode": "nullable"},
            {"name": "income_125000_149999", "type": "float", "mode": "nullable"},
            {"name": "income_150000_199999", "type": "float", "mode": "nullable"},
            {"name": "income_15000_19999", "type": "float", "mode": "nullable"},
            {"name": "income_200000_or_more", "type": "float", "mode": "nullable"},
            {"name": "income_20000_24999", "type": "float", "mode": "nullable"},
            {"name": "income_25000_29999", "type": "float", "mode": "nullable"},
            {"name": "income_30000_34999", "type": "float", "mode": "nullable"},
            {"name": "income_35000_39999", "type": "float", "mode": "nullable"},
            {"name": "income_40000_44999", "type": "float", "mode": "nullable"},
            {"name": "income_45000_49999", "type": "float", "mode": "nullable"},
            {"name": "income_50000_59999", "type": "float", "mode": "nullable"},
            {"name": "income_60000_74999", "type": "float", "mode": "nullable"},
            {"name": "income_75000_99999", "type": "float", "mode": "nullable"},
            {"name": "income_less_10000", "type": "float", "mode": "nullable"},
            {"name": "income_per_capita", "type": "float", "mode": "nullable"},
            {"name": "less_one_year_college", "type": "float", "mode": "nullable"},
            {
                "name": "less_than_high_school_graduate",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "male_10_to_14", "type": "float", "mode": "nullable"},
            {"name": "male_15_to_17", "type": "float", "mode": "nullable"},
            {"name": "male_18_to_19", "type": "float", "mode": "nullable"},
            {"name": "male_20", "type": "float", "mode": "nullable"},
            {"name": "male_21", "type": "float", "mode": "nullable"},
            {"name": "male_22_to_24", "type": "float", "mode": "nullable"},
            {"name": "male_25_to_29", "type": "float", "mode": "nullable"},
            {"name": "male_30_to_34", "type": "float", "mode": "nullable"},
            {"name": "male_35_to_39", "type": "float", "mode": "nullable"},
            {"name": "male_40_to_44", "type": "float", "mode": "nullable"},
            {
                "name": "male_45_64_associates_degree",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "male_45_64_bachelors_degree",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "male_45_64_grade_9_12", "type": "float", "mode": "nullable"},
            {"name": "male_45_64_graduate_degree", "type": "float", "mode": "nullable"},
            {"name": "male_45_64_high_school", "type": "float", "mode": "nullable"},
            {
                "name": "male_45_64_less_than_9_grade",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "male_45_64_some_college", "type": "float", "mode": "nullable"},
            {"name": "male_45_to_49", "type": "float", "mode": "nullable"},
            {"name": "male_45_to_64", "type": "float", "mode": "nullable"},
            {"name": "male_50_to_54", "type": "float", "mode": "nullable"},
            {"name": "male_55_to_59", "type": "float", "mode": "nullable"},
            {"name": "male_5_to_9", "type": "float", "mode": "nullable"},
            {"name": "male_60_to_61", "type": "float", "mode": "nullable"},
            {"name": "male_62_to_64", "type": "float", "mode": "nullable"},
            {"name": "male_65_to_66", "type": "float", "mode": "nullable"},
            {"name": "male_67_to_69", "type": "float", "mode": "nullable"},
            {"name": "male_70_to_74", "type": "float", "mode": "nullable"},
            {"name": "male_75_to_79", "type": "float", "mode": "nullable"},
            {"name": "male_80_to_84", "type": "float", "mode": "nullable"},
            {"name": "male_85_and_over", "type": "float", "mode": "nullable"},
            {"name": "male_male_households", "type": "float", "mode": "nullable"},
            {"name": "male_pop", "type": "float", "mode": "nullable"},
            {"name": "male_under_5", "type": "float", "mode": "nullable"},
            {
                "name": "management_business_sci_arts_employed",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "married_households", "type": "float", "mode": "nullable"},
            {"name": "masters_degree", "type": "float", "mode": "nullable"},
            {"name": "median_age", "type": "float", "mode": "nullable"},
            {"name": "median_income", "type": "float", "mode": "nullable"},
            {"name": "median_rent", "type": "float", "mode": "nullable"},
            {
                "name": "median_year_structure_built",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "million_dollar_housing_units",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "mobile_homes", "type": "float", "mode": "nullable"},
            {"name": "mortgaged_housing_units", "type": "float", "mode": "nullable"},
            {"name": "no_car", "type": "float", "mode": "nullable"},
            {"name": "no_cars", "type": "float", "mode": "nullable"},
            {"name": "nonfamily_households", "type": "float", "mode": "nullable"},
            {"name": "not_hispanic_pop", "type": "float", "mode": "nullable"},
            {"name": "not_in_labor_force", "type": "float", "mode": "nullable"},
            {"name": "not_us_citizen_pop", "type": "float", "mode": "nullable"},
            {"name": "occupation_management_arts", "type": "float", "mode": "nullable"},
            {
                "name": "occupation_natural_resources_construction_maintenance",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "occupation_production_transportation_material",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "occupation_sales_office", "type": "float", "mode": "nullable"},
            {"name": "occupation_services", "type": "float", "mode": "nullable"},
            {"name": "occupied_housing_units", "type": "float", "mode": "nullable"},
            {"name": "one_car", "type": "float", "mode": "nullable"},
            {
                "name": "one_parent_families_with_young_children",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "one_year_more_college", "type": "float", "mode": "nullable"},
            {"name": "other_race_pop", "type": "float", "mode": "nullable"},
            {
                "name": "owner_occupied_housing_units",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "owner_occupied_housing_units_lower_value_quartile",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "owner_occupied_housing_units_median_value",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "owner_occupied_housing_units_upper_value_quartile",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "percent_income_spent_on_rent",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "pop_16_over", "type": "float", "mode": "nullable"},
            {"name": "pop_25_64", "type": "float", "mode": "nullable"},
            {"name": "pop_25_years_over", "type": "float", "mode": "nullable"},
            {"name": "pop_5_years_over", "type": "float", "mode": "nullable"},
            {
                "name": "pop_determined_poverty_status",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "pop_in_labor_force", "type": "float", "mode": "nullable"},
            {"name": "population_1_year_and_over", "type": "float", "mode": "nullable"},
            {"name": "population_3_years_over", "type": "float", "mode": "nullable"},
            {"name": "poverty", "type": "float", "mode": "nullable"},
            {"name": "rent_10_to_15_percent", "type": "float", "mode": "nullable"},
            {"name": "rent_15_to_20_percent", "type": "float", "mode": "nullable"},
            {"name": "rent_20_to_25_percent", "type": "float", "mode": "nullable"},
            {"name": "rent_25_to_30_percent", "type": "float", "mode": "nullable"},
            {"name": "rent_30_to_35_percent", "type": "float", "mode": "nullable"},
            {"name": "rent_35_to_40_percent", "type": "float", "mode": "nullable"},
            {"name": "rent_40_to_50_percent", "type": "float", "mode": "nullable"},
            {"name": "rent_burden_not_computed", "type": "float", "mode": "nullable"},
            {"name": "rent_over_50_percent", "type": "float", "mode": "nullable"},
            {"name": "rent_under_10_percent", "type": "float", "mode": "nullable"},
            {
                "name": "renter_occupied_housing_units_paying_cash_median_gross_rent",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "sales_office_employed", "type": "float", "mode": "nullable"},
            {
                "name": "some_college_and_associates_degree",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "speak_only_english_at_home", "type": "float", "mode": "nullable"},
            {"name": "speak_spanish_at_home", "type": "float", "mode": "nullable"},
            {
                "name": "speak_spanish_at_home_low_english",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "three_cars", "type": "float", "mode": "nullable"},
            {"name": "total_pop", "type": "float", "mode": "nullable"},
            {"name": "two_cars", "type": "float", "mode": "nullable"},
            {"name": "two_or_more_races_pop", "type": "float", "mode": "nullable"},
            {
                "name": "two_parent_families_with_young_children",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "two_parents_father_in_labor_force_families_with_young_children",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "two_parents_in_labor_force_families_with_young_children",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "two_parents_mother_in_labor_force_families_with_young_children",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "two_parents_not_in_labor_force_families_with_young_children",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "unemployed_pop", "type": "float", "mode": "nullable"},
            {"name": "vacant_housing_units", "type": "float", "mode": "nullable"},
            {
                "name": "vacant_housing_units_for_rent",
                "type": "float",
                "mode": "nullable",
            },
            {
                "name": "vacant_housing_units_for_sale",
                "type": "float",
                "mode": "nullable",
            },
            {"name": "walked_to_work", "type": "float", "mode": "nullable"},
            {"name": "white_including_hispanic", "type": "float", "mode": "nullable"},
            {"name": "white_male_45_54", "type": "float", "mode": "nullable"},
            {"name": "white_male_55_64", "type": "float", "mode": "nullable"},
            {"name": "white_pop", "type": "float", "mode": "nullable"},
            {"name": "worked_at_home", "type": "float", "mode": "nullable"},
            {"name": "workers_16_and_over", "type": "float", "mode": "nullable"},
        ],
    )

    cbsa_2019_1yr_transform_csv >> load_cbsa_2019_1yr_to_bq
